from contextlib import contextmanager
import random

@contextmanager
def seed(data):
	"""Context manager for performing calculations pseudorandomly.
	
	If the same seed is given, the same random numbers should be generated.
	Only works with Python's built-in random library.
	"""
	state = random.getstate()
	random.seed(data)
	yield
	random.setstate(state)
