"""Test aspects related to the appearance of objects."""

import pytest

from dbecs.entity import Entity
from hyperbolic.block import Blocky
from hyperbolic.chunk import _chunk_size
from hyperbolic.coordinate import origin
from hyperbolic.position import Positioned
from hyperbolic.running import initialize
from hyperbolic.size import Sized

def test_small_blocks():
	"""Fill a chunk with blocks and look at their appearance."""
	world = initialize([])
	width = 10
	height = 8
	appearance = '.'

	for x in range(0, width):
		for y in range(0, height):
			block = Entity()
			block.add_component(Positioned, origin, x, y, 0, 0)
			block.add_component(Sized, 1, 1, 1)
			block.add_component(Blocky, appearance)
			world.place_block(block)

	for layer in world.get_range(origin, 0, 0, 0, width, height, 1):
		for row in layer:
			for block in row:
				assert Blocky(block).appearance == appearance

def test_larger_block():
	"""Place a block with a larger size in the world."""
	world = initialize([])
	x = 4
	y = 4
	z = 4
	size = 2

	block = Entity()
	block.add_component(Positioned, origin, x, y, z, 0)
	block.add_component(Sized, size, size, size)
	world.place_block(block)

	for x2 in range(x, x + size):
		for y2 in range(y, y + size):
			for z2 in range(z, z + size):
				assert world.get_block(origin, x2, y2, z2) == block
